@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -25,16 +25,56 @@
  *
  ****************************************************************************/
 
-#ifndef LIBKONOHA
-#define LIBKONOHA
+#ifndef KONOHA_H_
+#define KONOHA_H_
 
-#define KNHAPI_IMPORTS 1
-#include "konoha/konoha_dev.h"
+#ifdef HAVE_CONFIG_H
+#include"../config.h"
+#endif
 
-#undef KNH_SETv
-#undef KNH_FINALv
+/* configuration and dependency */
+#if defined(__APPLE__) && defined(KERNEL)
+#define KONOHA_ON_LKM
+#define KONOHA_ON_KEXT
+#endif
 
-#define KNH_SETv(ctx, v, o)     konoha_SETv(ctx, (Object**)(&v), UP(o))
-#define KNH_FINALv(ctx, v)      konoha_FINALv(ctx, (Object**)(&v))
+#ifndef KONOHA_ON_LKM
+#include<stdio.h>
+#include<stdlib.h>
+#include<ctype.h>
+#include<assert.h>
+#include<string.h>
+#include<setjmp.h>
+#endif
+
+#include"konoha/konoha_config.h"
+
+
+#include<stdarg.h>
+
+#include"konoha/konoha_tb.h"
+#ifdef KONOHA_ON_LKM
+#include"konoha/konoha_lkm.h"
+#endif
+
+#include"konoha/konoha_deps.h"
+
+/* types, symbols */
+
+#include"konoha/konoha_t.h"
+#include"konoha/konoha_class.h"
+
+/* functionality */
+
+#include"konoha/konoha_glue.h"
+#include"konoha/konoha_gc.h"
+#include"konoha/konoha_vm.h"
+#include"konoha/konoha_macros.h"
+#include"konoha/konoha_debug.h"
+
+/* konohaapi, prototypes */
+
+#include"konoha/konoha_api.h"
+
+#endif /*KONOHA_H_*/
 
-#endif /*LIBKONOHA*/
