@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -35,6 +35,30 @@ extern "C" {
 #endif
 
 /* ------------------------------------------------------------------------ */
+/* new version */
+
+#define Boolean_to(T, a)         ((T)a.bvalue)
+#define Int_to(T, a)             ((T)a.ivalue)
+#define IntNull_to(T, a, def)    ((T)(IS_bInt(a.o) ? (a.ivalue) : (def)))
+#define Float_to(T, a)           ((T)a.fvalue)
+#define FloatNull_to(T, a, def)  ((T)(IS_bFloat(a.o) ? (a.fvalue) : (def)))
+#define String_to(T, a)          ((T)knh_String_text(ctx, a.s))
+#define StringNull_to(T, a, def) ((T)(IS_bString(a.o) ? knh_String_text(ctx, a.s) : def))
+#define Glue_to(T, a)            ((T)((a.glue)->ptr))
+#define GlueNull_to(T, a, def)   (IS_bGlue(a.o) ? ((T)((a.glue)->ptr)) : (def))
+
+#define Boolean_to_(a)         (a.bvalue)
+#define Int_to_(a)             (a.ivalue)
+#define IntNull_to_(a, def)    ((IS_bInt(a.o) ? (a.ivalue) : (def)))
+#define Float_to_(a)           (a.fvalue)
+#define FloatNull_to_(a, def)  ((IS_bFloat(a.o) ? (a.fvalue) : (def)))
+#define String_to_(a)          (knh_String_text(ctx, a.s))
+#define StringNull_to_(a, def) ((IS_bString(a.o) ? knh_String_text(ctx, a.s) : def))
+#define Glue_to_(a)            (((a.glue)->ptr))
+#define GlueNull_to_(a, def)   (IS_bGlue(a.o) ? (((a.glue)->ptr)) : (def))
+
+/* ------------------------------------------------------------------------ */
+/* old */
 
 #define p_size(a)      ((size_t)(a).ivalue)
 #define p_int(a)         ((knh_int_t)(a).ivalue)
@@ -43,9 +67,13 @@ extern "C" {
 #define p_uinteger(a)    ((knh_uint_t)(a).ivalue)
 
 #define p_float(a)       ((a).fvalue)
+#ifdef KONOHA_ON_LKM
+#define p_double(a)      ((a).fvalue)
+#else
 #define p_double(a)      ((double)(a).fvalue)
+#endif
 #define p_bool(a)        ((a).bvalue)
-#define p_char(a)        (knh_String_tochar(a.s))
+#define p_char(a)        (__tochar(a.s))
 #define p_bytes(a)       (knh_Bytes_tobytes(a.s))
 #define p_cid(a)         ((a.c)->cid)
 #define ARG_Object_cid(a)  knh_Object_cid(a.o)
@@ -69,9 +97,8 @@ extern "C" {
 
 /* ------------------------------------------------------------------------ */
 
-
 #define KNH_RETURN(ctx, sfp, v) {\
-		Int *n_ = (Int*)v;\
+		knh_Int_t *n_ = (knh_Int_t*)v;\
 		KNH_NGCMOV(ctx, sfp[-1].o, n_);\
 		sfp[-1].data = (n_)->n.data;\
 		return; \
@@ -112,9 +139,15 @@ extern "C" {
 /* --------------------------------------------------------------------------- */
 /* [Iterator] */
 
+#define KNH_ITREND(ctx, sfp, n) {\
+		knh_Iterator_close(ctx, sfp[0].it);\
+		/*KNH_MOV(ctx, sfp[n].o, KNH_VOID); */\
+		return 0; \
+	}\
+
 #define KNH_ITRNEXT(ctx, sfp, i, v) {\
 		KNH_MOV(ctx, sfp[i].o, v);\
-		sfp[i].data = ((Int*)v)->n.data;\
+		sfp[i].data = ((knh_Int_t*)v)->n.data;\
 		return 1; \
 	}\
 
@@ -124,22 +157,41 @@ extern "C" {
 		return 1; \
 	}\
 
+#define KNH_ITRNEXT_IntX(ctx, sfp, i, n, def) {\
+		KNH_MOV(ctx, sfp[i].o, def);\
+		sfp[i].ivalue = n;\
+		return 1; \
+	}\
+
 #define KNH_ITRNEXT_Float(ctx, sfp, i, n) {\
 		KNH_MOV(ctx, sfp[i].o, KNH_FLOAT0);\
 		sfp[i].fvalue = n;\
 		return 1; \
 	}\
 
-#define KNH_ITREND(ctx, sfp, n) {\
-		knh_Iterator_close(ctx, sfp[0].it);\
-		KNH_MOV(ctx, sfp[n].o, KNH_VOID);\
-		return 0; \
+#define KNH_ITRNEXT_FloatX(ctx, sfp, i, n, def) {\
+		KNH_MOV(ctx, sfp[i].o, def);\
+		sfp[i].fvalue = n;\
+		return 1; \
 	}\
 
-#define HAS_ITRNEXT(v)   IS_NOTNULL(v)
+#define KNH_ITRNEXT_envsfp(ctx, sfp, i, envsfp) {\
+		KNH_MOV(ctx, sfp[i].o, envsfp[0].o);\
+		sfp[i].data = envsfp[0].data;\
+		return 1; \
+	}\
+
+
+//#define HAS_ITRNEXT(v)   IS_NOTNULL(v)
+
+/* --------------------------------------------------------------------------- */
+/* [Mapper] */
+
+
+#define KNH_GETMAPPER(ctx, sfp) sfp[1].mpr
 
 #define KNH_MAPPED(ctx, sfp, v) {\
-		Int *vn_ = (Int*)(v);\
+		knh_Int_t *vn_ = (knh_Int_t*)(v);\
 		KNH_MOV(ctx, sfp[0].o, vn_);\
 		sfp[0].data = (vn_)->n.data;\
 		return; \
