@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -28,6 +28,9 @@
 #ifndef KONOHA_T_H_
 #define KONOHA_T_H_
 
+#include"konoha_config.h"
+
+#ifdef KNH_USING_STDC
 #include<stdio.h>
 #include<limits.h>
 #include<float.h>
@@ -38,7 +41,7 @@
 #define __STDC_FORMAT_MACROS
 #include<inttypes.h>
 
-#include"konoha_config.h"
+#endif
 
 #if defined(KNH_USING_PTHREAD)
 	#include<pthread.h>
@@ -50,7 +53,12 @@ extern "C" {
 
 /* ------------------------------------------------------------------------ */
 
+#ifdef KONOHA_ON_WINDOWS
+#define METHOD             void __declspec(dllexport)
+#else
 #define METHOD             void  KNH_CC_FASTCALL
+#endif
+
 #define MAPPER             METHOD
 #define ITRNEXT            int   KNH_CC_FASTCALL
 
@@ -62,21 +70,8 @@ extern "C" {
 
 #define KONOHA_SYSTEM_BIT        (sizeof(void*) * CHAR_BIT))
 
-/* ------------------------------------------------------------------------ */
-/* Bool(ean), knh_bool_t */
-/* ------------------------------------------------------------------------ */
-
-typedef int                      knh_bool_t;
-typedef int                      knh_boolean_t;
-
-/* ------------------------------------------------------------------------ */
-/* Int, knh_int_t */
-/* ------------------------------------------------------------------------ */
-
 typedef int16_t           knh_int16_t;
 typedef uint16_t          knh_uint16_t;
-typedef int16_t           knh_short_t;
-typedef uint16_t          knh_ushort_t;
 typedef int32_t           knh_int32_t;
 typedef uint32_t          knh_uint32_t;
 typedef int64_t           knh_int64_t;
@@ -86,19 +81,27 @@ typedef intptr_t                  knh_intptr_t;
 typedef uintptr_t                 knh_uintptr_t;
 typedef knh_intptr_t              knh_index_t;
 
-#define KNH_INTPTR_FMT               "%"PRIdPTR
+#if defined(__LP64__)
+typedef int32_t           knh_short_t;
+typedef uint32_t          knh_ushort_t;
+#else
+typedef int16_t           knh_short_t;
+typedef uint16_t          knh_ushort_t;
+#endif
 
-//OLD
-//#define KNH_INTPTR_FMTX              "%"PRIxPTR
-//#define KNH_INT_BSIZE             (sizeof(knh_int_t))
-//#define KNH_UINT_FMT              "%lu"
-//typedef size_t                    knh_size_t;
-//typedef size_t                    bsize_t;
-//typedef knh_intptr_t                 int_byte_t;
-//#define KNH_INTPTR_MAX               LONG_MAX
-//#define KNH_INTPTR_MIN               LONG_MIN
-//#define KNH_UINTPTR_MAX              ULONG_MAX
-//#define KNH_UINTPTR_MIN              0
+/* ------------------------------------------------------------------------ */
+/* Bool(ean), knh_bool_t */
+/* ------------------------------------------------------------------------ */
+
+typedef knh_intptr_t      knh_bool_t;
+typedef knh_intptr_t      knh_boolean_t;
+
+/* ------------------------------------------------------------------------ */
+/* Int, knh_int_t */
+/* ------------------------------------------------------------------------ */
+
+#define KNH_INTPTR_FMT               "%"PRIdPTR
+#define KNH_INTPTR_UFMT              "%"PRIuPTR
 
 /* ------------------------------------------------------------------------ */
 /* Integer, knh_int_t */
@@ -106,12 +109,11 @@ typedef knh_intptr_t              knh_index_t;
 
 #ifdef KNH_USING_INT32
 
-typedef int32_t        knh_int_t;
-typedef uint32_t       knh_uint_t;
+typedef long                knh_int_t;
+typedef unsigned long       knh_uint_t;
 
 #define KNH_INT_MAX               LONG_MAX
-#define KNH_INT_MIN               (LONG_MIN + 1)
-#define KNH_INT_NULL              (LONG_MIN)
+#define KNH_INT_MIN               LONG_MIN
 #define KNH_INT_FMT               "%ld"
 #define KNH_INT_XFMT              "%lx"
 #define KNH_UINT_MAX              ULONG_MAX
@@ -134,8 +136,7 @@ typedef knh_uint64_t       knh_uint_t;
 #endif
 
 #define KNH_INT_MAX               LLONG_MAX
-#define KNH_INT_MIN               (LLONG_MIN + 1)
-#define KNH_INT_NULL              (LLONG_MIN)
+#define KNH_INT_MIN               LLONG_MIN
 #define KNH_INT_FMT               "%lld"
 #define KNH_INT_XFMT              "%llx"
 #define KNH_UINT_MAX              ULLONG_MAX
@@ -147,14 +148,33 @@ typedef knh_uint64_t       knh_uint_t;
 
 #define KNH_INT_FMTSIZ            40
 
-//OLD
-//#define KNH_INT_BSIZE             (sizeof(knh_int_t))
-
 /* ------------------------------------------------------------------------ */
 /* Float, knh_float_t */
 /* ------------------------------------------------------------------------ */
 
-#ifdef KNH_USING_LONGDOUBLE
+#ifdef KNH_USING_NOFLOAT
+/* dummy */
+typedef int               knh_float_t;
+#define KNH_FLOAT_MAX             0
+#define KNH_FLOAT_MIN             0
+#define KNH_FLOAT_STEP            0
+#define KNH_FLOAT_FMT             "%d"
+#define KNH_FLOAT_FMT1            "%d"
+#define KNH_FLOAT_FMT2            "%d"
+#define KNH_FLOAT_FMT3            "%d"
+#define KNH_FLOAT_FMT4            "%d"
+#define KNH_FLOAT_FMTE            "%d"
+#define KNH_FLOAT_NAN             0
+#define KNH_FLOAT_NULL            KNH_FLOAT_NAN
+#define KNH_FLOAT_FMTSIZ          40
+#define KNH_FLOAT_ZERO            0
+#define KNH_FLOAT_ONE             1
+#else
+#define KNH_FLOAT_ZERO            0.0
+#define KNH_FLOAT_ONE             1.0
+#endif/*KNH_USING_NOFLOAT*/
+
+#if defined(KNH_USING_LONGDOUBLE) && !defined(KNH_FLOAT_FMT)
 typedef long double               knh_float_t;
 #define KNH_FLOAT_MAX             LDBL_MAX
 #define KNH_FLOAT_MIN             (-(LDBL_MAX))
@@ -165,9 +185,9 @@ typedef long double               knh_float_t;
 #define KNH_FLOAT_FMT3            "%.3Lf"
 #define KNH_FLOAT_FMT4            "%.4Lf"
 #define KNH_FLOAT_FMTE            "%Le"
-#else/*KNH_USING_LONGDOUBLE*/
+#endif
 
-#ifdef KNH_USING_FLOAT
+#if defined(KNH_USING_FLOAT) && !defined(KNH_FLOAT_FMT)
 typedef float                     knh_float_t;
 #define KNH_FLOAT_MAX             FLT_MAX
 #define KNH_FLOAT_MIN             (-(FLT_MAX))
@@ -178,8 +198,9 @@ typedef float                     knh_float_t;
 #define KNH_FLOAT_FMT3            "%.3f"
 #define KNH_FLOAT_FMT4            "%.4f"
 #define KNH_FLOAT_FMTE            "%e"
-#else/*KNH_USING_FLOAT*/
+#endif
 
+#ifndef KNH_FLOAT_FMT
 typedef double                    knh_float_t;
 #define KNH_FLOAT_MAX             DBL_MAX
 #define KNH_FLOAT_MIN             (-(DBL_MAX))
@@ -190,12 +211,13 @@ typedef double                    knh_float_t;
 #define KNH_FLOAT_FMT3            "%.3f"
 #define KNH_FLOAT_FMT4            "%.4f"
 #define KNH_FLOAT_FMTE            "%e"
-#endif/*KNH_USING_FLOAT*/
-#endif/*KNH_USING_LONGDOUBLE*/
+#endif
 
+#ifndef KNH_FLOAT_NAN
 #define KNH_FLOAT_NAN             (0.0/0.0)
 #define KNH_FLOAT_NULL            KNH_FLOAT_NAN
 #define KNH_FLOAT_FMTSIZ          80
+#endif /*KNH_FLOAT_NAN*/
 
 /* ------------------------------------------------------------------------ */
 /* String, knh_uchar_t */
@@ -210,6 +232,11 @@ typedef struct {
 
 #define ismulti(c)             (((knh_uchar_t)c)>127)
 
+#define B(c)     new_bytes(c)
+#define B2(c,n)   new_bytes__2(c,n)
+#define STEXT(c)  new_bytes__2(c,sizeof(c)-1)
+#define ISB(t,c) (t.len == (sizeof(c)-1) && knh_strncmp((char*)t.buf,c,t.len) == 0)
+
 /* ------------------------------------------------------------------------ */
 /* ConstData  */
 /* ------------------------------------------------------------------------ */
@@ -229,6 +256,11 @@ typedef struct {
 	char *value;
 } knh_StringConstData_t;
 
+typedef struct {
+	char *name;
+	void *ptr;
+} knh_NamedPointerData_t;
+
 /* ------------------------------------------------------------------------ */
 /* knh_flag_t */
 /* ------------------------------------------------------------------------ */
@@ -267,25 +299,58 @@ typedef knh_uint16_t       knh_class_t;   /* class id */
 typedef knh_uint16_t       knh_type_t;    /* extended knh_type_t */
 typedef knh_uint16_t       knh_expt_t;    /* knh_expt_t */
 
-#define KNH_FLAG_TF_NN                KNH_FLAG_T0
-#define TYPE_ISNULLABLE(t)            ((t & KNH_FLAG_TF_NN)==0)
-#define IS_NNTYPE(t)                  ((t & KNH_FLAG_TF_NN)==KNH_FLAG_TF_NN)
-#define NNTYPE_cid(c)                 (c|KNH_FLAG_TF_NN)
-#define TYPE_TONNTYPE(t)              (t|KNH_FLAG_TF_NN)
+/* knh_struct_t */
+#define KNH_FLAG_SF_FIELD          KNH_FLAG_T1
 
-#define CLASS_type(t)                (t&(~KNH_FLAG_TF_NN))
+#define STRUCT_ISFIELD(sid)        ((sid & KNH_FLAG_SF_FIELD) == KNH_FLAG_SF_FIELD)
+#define BSIZE_TOSID(bsize)         (((knh_struct_t)bsize)|KNH_FLAG_SF_FIELD)
+#define STRUCT_FIELD(bsize)        (((knh_struct_t)bsize)|KNH_FLAG_SF_FIELD)
+#define STRUCT_FIELDSIZE(sid)      (sid & (~KNH_FLAG_SF_FIELD))
+#define STRUCT_UNMASK(sid)         (sid & (~KNH_FLAG_SF_FIELD))
+#define KNH_ASSERT_sid(sid)        KNH_ASSERT(((knh_struct_t)sid) < ctx->share->StructTableSize)
+
+/* knh_class_t */
+
+#define CLASS_newid                ((knh_class_t)-1)
+#define CLASS_unknown              ((knh_class_t)-2)
+
+#define DBG2_ASSERT_cid(cid)        KNH_ASSERT(cid < KNH_TCLASS_SIZE)
+#define CLASSN(cid)                knh_ClassTable_CLASSN(ctx, cid)
+#define CLASSNo(o)                 knh_ClassTable_CLASSN(ctx, knh_Object_cid(o))
+#define CTXCLASSN(cid)     knh_Context_CLASSN(ctx,cid)
+//#define CTXTYPEQN(cid)     knh_Context_CLASSN(ctx,CLASS_type(cid)), TYPEQ(cid)
+
+/* knh_type_t */
+
+#define KNH_FLAG_TF_NA      KNH_FLAG_T0
+#define CLASS_type(t)       (t&(~KNH_FLAG_TF_NA))
+#define IS_NNTYPE(t)        ((t & KNH_FLAG_TF_NA)==0)
+#define IS_NATYPE(t)        ((t & KNH_FLAG_TF_NA)==KNH_FLAG_TF_NA)
+#define NATYPE_cid(t)       (t|KNH_FLAG_TF_NA)
+#define NNTYPE_cid(t)       (t)
+
+#define IS_ubxint(t)        (IS_NNTYPE(t) && ClassTable(CLASS_type(t)).bcid == CLASS_Int)
+#define IS_ubxfloat(t)      (IS_NNTYPE(t) && ClassTable(CLASS_type(t)).bcid == CLASS_Float)
+#define IS_ubxboolean(t)    (NNTYPE_Boolean == t)
+#define IS_ubxtype(t)       (IS_NNTYPE(t) && (ClassTable(CLASS_type(t)).bcid == CLASS_Int || ClassTable(CLASS_type(t)).bcid == CLASS_Float || t == NNTYPE_Boolean) )
+#define IS_bxint(t)         (IS_NATYPE(t) && ClassTable(CLASS_type(t)).bcid == CLASS_Int)
+#define IS_bxfloat(t)       (IS_NATYPE(t) && ClassTable(CLASS_type(t)).bcid == CLASS_Float)
 
 // @NOUSE
+#define TYPEN(type)                   knh_TYPEN(ctx,type)
 #define TYPEQN(t)                     TYPEN(t), TYPEQ(t)
 
-#define TYPE_var                      CLASS_Nue
-#define TYPE_void                     NNTYPE_cid(CLASS_Nue)
-#define NNTYPE_void                   NNTYPE_cid(CLASS_Nue)
-#define TYPE_any                      CLASS_Any
-#define NNTYPE_any                    CLASS_any
+#define TYPE_var                      CLASS_Tvar
+#define TYPE_void                     CLASS_Tvoid
+//#define TYPE_Void                     TYPE_void
+
+/* knh_expt_t */
+
+#define EXPT_unknown  ((knh_expt_t)-1)
+#define EXPT_newid    ((knh_expt_t)0)
+#define KNH_ASSERT_eid(eid)    KNH_ASSERT(eid < ctx->share->ExptTableSize + 1)
+#define EXPTN(eid)   __tochar(knh_getExptName(ctx, eid))
 
-//OLD
-//typedef knh_ushort_t          knh_nsid_t;
 
 /* ------------------------------------------------------------------------ */
 
@@ -323,7 +388,19 @@ typedef knh_ushort_t          knh_methodn_t;
 
 #define METHODN_TOFIELDN(mn)     (mn & KNH_FLAG_MN_FIELDN)
 
-/* ------------------------------------------------------------------------ */
+#define METHODN_LAMBDA          FIELDN_
+
+#define FIELDN(fn) __tochar(knh_getFieldName(ctx, fn))
+#define METHODN(mn) __tochar(knh_getFieldName(ctx, METHODN_TOFIELDN(mn)))
+
+/* src/kc/misc_.c */
+#define METHODN_opALT    METHODN_NONAME
+#define METHODN_opSame   METHODN_NONAME
+#define METHODN_opInto   METHODN_NONAME
+#define METHODN_opFrom   METHODN_NONAME
+#define METHODN_opWhere  METHODN_NONAME
+#define METHODN_opTo     METHODN_NONAME
+#define METHODN_opUntil  METHODN_NONAME
 
 /* ------------------------------------------------------------------------ */
 /* Object */
@@ -338,14 +415,14 @@ typedef knh_ushort_t          knh_methodn_t;
 typedef knh_ushort_t knh_lock_t;
 
 #ifdef KNH_OBJECT_MAGIC
-	#define KNH_ASSERT_ISOBJECT(o)        DBG2_ASSERT((o)->h.magic == KNH_OBJECT_MAGIC)
+	#define DBG2_ASSERT_ISOBJECT(o)        DBG2_ASSERT((o)->h.magic == KNH_OBJECT_MAGIC)
 #else/*KONOHA_OBJECT_MAGIC*/
-	#define KNH_ASSERT_ISOBJECT(o)
+	#define DBG2_ASSERT_ISOBJECT(o)
 #endif/*KONOHA_OBJECT_MAGIC*/
 
 typedef struct knh_hObject_t {
 	knh_ushort_t magic; knh_flag_t  flag;
-	knh_class_t bcid;   knh_class_t cid;
+	knh_class_t  bcid;  knh_class_t cid;
 	knh_ushort_t ctxid; knh_lock_t  lock;
 //#ifdef KNH_HOBJECT_REFC
 	KNH_MT_VOLATILE knh_uintptr_t refc;
@@ -366,22 +443,37 @@ typedef struct knh_Object_t {
 #define ObjectNULL      Object
 #define Any             knh_Object_t
 #define This            knh_Object_t
-#define Any1            knh_Object_t
-#define Any2            knh_Object_t
+#define T1              knh_Object_t
+#define T2              knh_Object_t
 #define UP(o)           (Object*)(o)
 
-#define KNH_FIELDn(v,n)            ((Script*)(v))->fields[(n)]
+#define KNH_FIELDn(v,n)            ((knh_Script_t*)(v))->fields[(n)]
 
 /* ------------------------------------------------------------------------ */
 /* Common Object Structure */
 
-typedef knh_ushort_t          knh_fileid_t;
-typedef knh_ushort_t          knh_sline_t;
+typedef knh_ushort_t              knh_uri_t;
+#define KNH_FLAG_URI_UNTRUSTED    KNH_FLAG_T0
+
+#define URI_UNMASK(uri)           (uri & (~(KNH_FLAG_URI_UNTRUSTED)))
+#define URI_TRUSTED(uri)          (uri & (~(KNH_FLAG_URI_UNTRUSTED)))
+#define URI_UNTRUSTED(uri)        (uri | KNH_FLAG_URI_UNTRUSTED)
+#define URI_ISTRUSTED(uri)        ((uri & KNH_FLAG_URI_UNTRUSTED) == 0)
+#define URI_EVAL                  URI_UNTRUSTED((knh_uri_t)0)
+
+#define NSN_main           0
+#define FILEN_unknown      0
+#define KNH_ASM_JMP(ctx, l)  KNH_ASM_JMP_(ctx, l);
+
+#define URIDN(uri) __tochar(knh_getResourceName(ctx, uri))
+#define FILEN(uri) knh_safefile(URIDN(uri))
+
+typedef knh_ushort_t              knh_sline_t;
 
 typedef struct {
 	knh_hObject_t h;
 	void *ref;
-	knh_fileid_t fileid;
+	knh_uri_t uri;
 	knh_sline_t  line;
 } knh_FileLine_t ;
 
@@ -390,62 +482,57 @@ typedef struct {
 	int   line;
 } knh_fline_t;
 
-#define knh_Object_data(o)       (((Int*)(o))->n.data)
+#define knh_Object_data(o)       (((knh_Int_t*)(o))->n.data)
 
 /* ------------------------------------------------------------------------ */
 /* Thread */
 /* ------------------------------------------------------------------------ */
 
-#define KNH_USING_NOTHREAD 1
-#ifdef KNH_USING_PTHREAD
-#undef KNH_USING_NOTHREAD
+#if defined(KNH_USING_PTHREAD)
 #define knh_thread_t pthread_t
 #define knh_thread_key_t pthread_key_t
 #define knh_mutex_t pthread_mutex_t
+#elif defined(KNH_USING_BTRON)
+#define knh_thread_t W
+#define knh_thread_key_t W
+#define knh_mutex_t W
+#else
+typedef knh_intptr_t knh_thread_t;
+typedef knh_intptr_t knh_thread_key_t;
+typedef knh_intptr_t knh_mutex_t;
 #endif
-
-#ifdef KNH_USING_NOTHREAD
-typedef knh_int_t knh_thread_t;
-typedef knh_int_t knh_thread_key_t;
-typedef knh_int_t knh_mutex_t;
-#endif
+typedef void *(*threadfunc_t)(void *);
 
 typedef struct knh_Thread_t {
 	knh_hObject_t h;
 	knh_thread_t thid;
 } knh_Thread_t ;
 
-
-#ifndef KNH_TLOCK_SIZE
-#define KNH_TLOCK_SIZE 256
-#endif
-
-#define SIZEOF_TLOCK       (KNH_TLOCK_SIZE * sizeof(knh_LockTable_t))
-
-typedef void (*knh_fmutexlock)(Ctx *ctx, knh_lock_t lock, Object *o);
-typedef void (*knh_fmutexunlock)(Ctx *ctx, knh_lock_t lock);
-
 typedef struct knh_LockTable_t {
-	knh_lock_t  lock;     knh_flag_t  flag;
-	knh_mutex_t mutex;
-	knh_fmutexlock       flock;
-	knh_fmutexunlock       funlock;
-	int ctxid;
+	knh_mutex_t *mutex;
 	size_t count;
 	struct knh_String_t *name;
 	union {
-		Object *sharedObject;
+		Object *so;   /* shared object*/
 		struct knh_LockTable_t *unused;
 	};
+	char *filename;
+	int   lineno;
 } knh_LockTable_t;
 
-#define LOCK_NO           ((knh_lock_t)0)
+#define LOCK_NOP          ((knh_lock_t)0)
 #define LOCK_MEMORY       ((knh_lock_t)1)
 #define LOCK_SYSTBL       ((knh_lock_t)2)
-#define LOCK_GIANT        ((knh_lock_t)3)
+#define LOCK_UNUSED       3
 
-#define KNH_LOCK(ctx, lk, o)    (ctx->share)->LockTable[lk].flock(ctx, lk, o)
-#define KNH_UNLOCK(ctx, lk, o)  (ctx->share)->LockTable[lk].funlock(ctx, lk)
+#ifndef KNH_TLOCK_SIZE
+#define KNH_TLOCK_SIZE (LOCK_UNUSED+1)
+#endif
+
+#define SIZEOF_TLOCK   (KNH_TLOCK_SIZE * sizeof(knh_LockTable_t))
+
+#define KNH_LOCK(ctx, lockid, o)    knh_lockID(ctx, lockid, o, (char*)__FILE__, (int)__LINE__)
+#define KNH_UNLOCK(ctx, lockid, o)  knh_unlockID(ctx, lockid, (char*)__FILE__, __LINE__)
 
 /* ------------------------------------------------------------------------ */
 /* Stack Frame Pointer */
@@ -462,18 +549,30 @@ typedef struct knh_sfp_t {
 		struct knh_Class_t  *c;
 		struct knh_String_t *s;
 		struct knh_Bytes_t  *ba;
+		struct knh_Pair_t   *pair;
+		struct knh_Tuple_t  *tuple;
+		struct knh_Range_t  *range;
+		struct knh_Array_t  *a;
+		struct knh_IArray_t  *ia;
+		struct knh_FArray_t  *fa;
 		struct knh_Iterator_t *it;
+		struct knh_DictMap_t  *dmap;
+		struct knh_DictSet_t  *dset;
+		struct knh_HashMap_t  *hmap;
+		struct knh_HashSet_t *hset;
 		struct knh_Closure_t *cc;
-		struct knh_Glue_t   *glue;
 		struct knh_OutputStream_t *w;
 		struct knh_Method_t *mtd;
 		struct knh_Mapper_t *mpr;
 		struct knh_Exception_t *e;
 		struct knh_ExceptionHandler_t *hdr;
+		struct knh_NameSpace_t *ns;
+		struct knh_Glue_t   *glue;
 	};
 	union {
 		knh_boolean_t bvalue;
 		knh_int_t     ivalue;
+		knh_uint_t    uvalue;
 		knh_float_t   fvalue;
 		knh_uint64_t  data;
 		knh_code_t    *pc;
@@ -498,22 +597,22 @@ typedef struct knh_sfp_t {
 #define STRUCT_newid         ((knh_struct_t)-1)
 #define SIZEOF_TSTRUCT       (KNH_TSTRUCT_SIZE * sizeof(knh_StructTable_t))
 
-char *konoha_geStructTableName(Ctx *ctx, knh_struct_t sid);
-#define STRUCTN(sid)         konoha_geStructTableName(ctx, sid)
+char *knh_getStructTableName(Ctx *ctx, knh_struct_t sid);
+#define STRUCTN(sid)         knh_getStructTableName(ctx, sid)
 
 typedef void (*knh_fgchook)(Ctx *ctx);
 typedef void (*knh_ftraverse)(Ctx *ctx, Object *);
 typedef int (*knh_finit)(Ctx *);
 
-typedef knh_uintptr_t                knh_hcode_t;  /* knh_hcode_t */
-#define knh_hcode_join(s1,s2)	   ((knh_hcode_t)s1 << (sizeof(knh_short_t)*8)) + s2;
+typedef knh_uintptr_t                knh_hashcode_t;  /* knh_hashcode_t */
+#define knh_hcode_join(s1,s2)	   ((knh_hashcode_t)s1 << (sizeof(knh_short_t)*8)) + s2;
 
 #define KNH_OBJECT_RAWINIT              (-1)
 
 typedef void         (*knh_fstruct_init)(Ctx *, Object *, int);
 typedef void         (*knh_fstruct_traverse)(Ctx *, Object*, knh_ftraverse);
-typedef int          (*knh_fstruct_compareTo)(Object*, Object*);
-typedef knh_hcode_t  (*knh_fstruct_hashCode)(Ctx *ctx, Object *);
+typedef int          (*knh_fstruct_compareTo)(Ctx *ctx, Object*, Object*);
+typedef knh_hashcode_t  (*knh_fstruct_hashCode)(Ctx *ctx, Object *);
 typedef Object*      (*knh_fstruct_copy)(Ctx *, Object *);
 typedef void         (*knh_fstruct_newClass)(Ctx *ctx, knh_class_t cid);
 typedef struct knh_String_t* (*knh_fstruct_getkey)(Ctx *ctx, knh_sfp_t *lsfp);
@@ -559,14 +658,17 @@ typedef struct {
 	struct knh_String_t       *sname;
 	struct knh_String_t       *lname;
 	struct knh_Class_t        *class_cid;
-	struct knh_ClassStruct_t  *cstruct;
+	struct knh_Class_t        *class_natype;
+	struct knh_ClassField_t  *cstruct;
 	struct knh_ClassMap_t     *cmap;
 	struct knh_Object_t       *cspec;
 	struct knh_DictMap_t      *constPool;
 	knh_fdefault               fdefault;
+#ifndef KNH_USING_NODATAPOOL
 	knh_mutex_t                dataLock;
 	struct knh_Array_t        *dataList;
-	struct knh_DictMap_t      *dataMap;
+	struct knh_DictMap_t      *dataKeyMap;
+#endif
 } knh_ClassTable_t;
 
 /* ------------------------------------------------------------------------ */
@@ -611,30 +713,17 @@ typedef struct {
 /* ------------------------------------------------------------------------ */
 /* [SystemShare] */
 
-#ifndef KNH_TINT_MIN
-#define KNH_TINT_MIN (-1)  /* @property */
-#endif
-
-#ifndef KNH_TINT_MAX
-#ifdef KNH_FASTMODE
-#define KNH_TINT_MAX 1024  /* @property */
-#else
-#define KNH_TINT_MAX 256  /* @property */
-#endif
-#endif
-
-#define SIZEOF_TINT (sizeof(knh_Object_t*) * (KNH_TINT_MAX - KNH_TINT_MIN + 1))
 #define SIZEOF_TSTRING (sizeof(knh_Object_t*) * KNH_TSTRING_SIZE)
 
-#define knh_systemEncoding    DP(ctx->sys)->enc
+#define KNH_ENC             DP(ctx->sys)->enc
 
 #define KNH_NULL            (ctx->share->constNull)
-#define KNH_VOID            (ctx->share->constVoid)
+#define KNH_VOID            (ctx->share->constNull)
 #define KNH_TRUE            (ctx->share->constTrue)
 #define KNH_FALSE           (ctx->share->constFalse)
-#define KNH_SYSTEM          (ctx->sys)
-#define KNH_INT0            (ctx->share->tInt[0 - KNH_TINT_MIN])
+#define KNH_INT0            (ctx->share->constInt0)
 #define KNH_FLOAT0          (ctx->share->constFloat0)
+#define KNH_SYSTEM          (ctx->sys)
 
 /* ------------------------------------------------------------------------ */
 
@@ -643,11 +732,11 @@ typedef struct {
 
 /* ------------------------------------------------------------------------ */
 
-#ifndef KNH_TOBJECTPAGE_SIZE
-#define KNH_TOBJECTPAGE_SIZE     1024
+#ifndef KNH_TOBJECTPAGE_INITSIZE
+#define KNH_TOBJECTPAGE_INITSIZE     1024
 #endif
 
-#define SIZEOF_TOBJECTPAGE       (sizeof(knh_ObjectPageTable_t) * KNH_TOBJECTPAGE_SIZE)
+//#define SIZEOF_TOBJECTPAGE       (sizeof(knh_ObjectPageTable_t) * KNH_TOBJECTPAGE_INITSIZE)
 
 #ifndef KNH_OBJECTPAGE_SIZE
 #define KNH_OBJECTPAGE_SIZE     4096
@@ -664,23 +753,42 @@ typedef struct knh_ObjectPageTable_t {
 
 typedef struct {
 	/* system table */
-	size_t              threadSize;
+	knh_ObjectPageTable_t    *ObjectPageTable;
+	const knh_StructTable_t  *StructTable;
+	const knh_ClassTable_t   *ClassTable;
+	const knh_ExptTable_t    *ExptTable;
+	size_t ObjectPageTableSize;
+	size_t ObjectPageTableMaxSize;
+	size_t StructTableSize;
+	size_t ClassTableSize;
+	size_t ExptTableSize;
+
+	/* system shared const */
+	knh_Object_t         *constNull;
+	knh_Object_t         *constTrue;
+	knh_Object_t         *constFalse;
+	struct knh_Int_t     *constInt0;
+	struct knh_Float_t   *constFloat0;
+	struct knh_String_t  **tString;
+	/* share level=2 */
+	struct knh_NameSpace_t   *mainns;
+
+	/* thread */
+	size_t              contextCounter;
+	size_t              threadCounter;
 	knh_LockTable_t    *LockTable;
 	knh_LockTable_t    *unusedLockTable;
-	knh_ObjectPageTable_t  *ObjectPageTable; size_t ObjectPageTableSize;
-	knh_StructTable_t  *StructTable; size_t StructTableSize;
-	knh_ClassTable_t   *ClassTable;  size_t ClassTableSize;
-	knh_ExptTable_t    *ExptTable;   size_t ExptTableSize;
+} knh_SharedData_t ;
 
-	/* system const */
-	knh_Object_t   *constNull;
-	knh_Object_t   *constVoid;
-	knh_Object_t   *constTrue;
-	knh_Object_t   *constFalse;
-	struct knh_Float_t   *constFloat0;
-	struct knh_Int_t     **tInt;
-	struct knh_String_t  **tString;
-} knh_ctxshare_t ;
+#define LockTable(mx)     ctx->share->LockTable[mx]
+#define pLockTable(mx)    (knh_LockTable_t*)(ctx->share->LockTable + (mx))
+
+#define StructTable(sid)  ctx->share->StructTable[sid]
+#define pStructTable(sid) (knh_StructTable_t*)(ctx->share->StructTable + (sid))
+#define ClassTable(cid)   ctx->share->ClassTable[cid]
+#define pClassTable(cid)  (knh_ClassTable_t*)(ctx->share->ClassTable + (cid))
+#define ExptTable(eid)    ctx->share->ExptTable[eid]
+#define pExptTable(eid)   (knh_ExptTable_t*)(ctx->share->ExptTable + (eid))
 
 /* ------------------------------------------------------------------------ */
 
@@ -701,15 +809,16 @@ typedef struct knh_Context_t {
 	struct knh_Method_t         **fmtCache;
 	struct knh_Mapper_t         **mprCache;
 
-	/* root table */
-	knh_ctxshare_t   *share;
-	knh_ctxstat_t    *stat;
+	/* shared table */
+	const knh_SharedData_t       *share;
+	knh_ctxstat_t                *stat;
 
 	knh_flag_t                   flag;
 	knh_ushort_t                 ctxid;
 	char*                        cwd;
-	knh_thread_t                 threadid;
-	struct knh_Context_t        *parent;
+	const struct knh_Context_t   *parent;
+	knh_mutex_t                  *ctxlock;
+	const struct knh_Context_t   *unusedContext;
 
 	struct knh_System_t*         sys;
 	struct knh_String_t*         enc;
@@ -720,13 +829,12 @@ typedef struct knh_Context_t {
 	struct knh_OutputStream_t*   bufw;
 	struct knh_Bytes_t*          bconvbuf;
 	struct knh_DictMap_t*        props;
+	struct knh_Gamma_t            *kc;
+	struct knh_Array_t          *lines;
 
-	struct knh_NameSpace_t      *ns;
-	struct knh_DictMap_t        *tsymbolDictMap;
-	struct knh_Asm_t            *cmpr;
+	int    hasError;
+	struct knh_String_t *msgError;
 
-//	struct knh_HashMap_t        *tmapperHashMap;
-//	struct knh_HashMap_t        *tmethodHashMap;
 } knh_Context_t ;
 
 /* ------------------------------------------------------------------------ */
@@ -818,7 +926,7 @@ typedef struct {
 
 typedef knh_intptr_t knh_io_t;
 
-typedef knh_io_t (*f_io_open)(Ctx *ctx, knh_bytes_t urn, char *mode);
+typedef knh_io_t (*f_io_open)(Ctx *ctx, knh_bytes_t urn, char *mode, int isThrowable);
 typedef void (*f_io_init)(Ctx *ctx, Object *stream, char *mode);
 typedef knh_intptr_t (*f_io_read)(Ctx *ctx, knh_io_t fd, char *buf, size_t bufsiz);
 typedef knh_intptr_t (*f_io_write)(Ctx *ctx, knh_io_t fd, char *buf, size_t bufsiz);
