@@ -405,7 +405,7 @@ static int knh_Float_compareTo(Ctx *ctx, knh_Float_t *o, knh_Float_t *o2)
 
 static void knh_String_init(Ctx *ctx, knh_String_t *s, int init)
 {
-	s->str = (knh_uchar_t*)""; // for safety
+	s->str = NULL;
 	s->size = 0;
 	s->orign = NULL;
 }
@@ -416,7 +416,7 @@ static void knh_String_traverse(Ctx *ctx, knh_String_t *s, knh_ftraverse ftr)
 		ftr(ctx, UP(s->orign));
 	}
 	else if(!knh_String_isTextSgm(s)) {
-		if(IS_SWEEP(ftr)) {
+		if(IS_SWEEP(ftr) && s->str) {
 			KNH_FREE(ctx, s->str, KNH_SIZE(s->size + 1));
 			s->str = NULL;
 		}
