@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -38,9 +38,9 @@ extern "C" {
 /* ======================================================================== */
 /* [Bytes] */
 
-KNHAPI(InputStream*) new_InputStream__io(Ctx *ctx, String *urn, knh_io_t fd, knh_iodrv_t *drv)
+KNHAPI(knh_InputStream_t*) new_InputStream__io(Ctx *ctx, knh_String_t *urn, knh_io_t fd, knh_iodrv_t *drv)
 {
-	InputStream* o = (InputStream*)new_Object_bcid(ctx, CLASS_InputStream, 0);
+	knh_InputStream_t* o = (knh_InputStream_t*)new_Object_bcid(ctx, CLASS_InputStream, 0);
 	KNH_SETv(ctx, DP(o)->urn, urn);
 	DP(o)->fd = fd;
 	if(fd != -1) {
@@ -55,15 +55,16 @@ KNHAPI(InputStream*) new_InputStream__io(Ctx *ctx, String *urn, knh_io_t fd, knh
 		}
 		DP(o)->bufpos = 0;
 		DP(o)->bufend = 0;  /* empty */
+		DP(o)->driver->finit(ctx, (Object*)o, "r");
 	}
 	return o;
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(InputStream*) new_InputStream__FILE(Ctx *ctx, String *urn, FILE *fp, knh_iodrv_t *drv)
+KNHAPI(knh_InputStream_t*) new_InputStream__FILE(Ctx *ctx, knh_String_t *urn, FILE *fp, knh_iodrv_t *drv)
 {
-	InputStream* o = (InputStream*)new_Object_bcid(ctx, CLASS_InputStream, 0);
+	knh_InputStream_t* o = (knh_InputStream_t*)new_Object_bcid(ctx, CLASS_InputStream, 0);
 	KNH_SETv(ctx, DP(o)->urn, urn);
 	if(fp != NULL) {
 		DP(o)->fp = fp;
@@ -76,20 +77,20 @@ KNHAPI(InputStream*) new_InputStream__FILE(Ctx *ctx, String *urn, FILE *fp, knh_
 /* ======================================================================== */
 /* [methods] */
 
-Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn, String *m)
+Object *knh_InputStream_open(Ctx *ctx, knh_InputStream_t *o, knh_String_t *urn, knh_String_t *m)
 {
-	knh_bytes_t fname = knh_String_tobytes(urn);
+	knh_bytes_t fname = __tobytes(urn);
 	knh_index_t loc = knh_bytes_index(fname, ':');
 	if(loc == -1 || (loc == 1 && isalpha(fname.buf[0]))) {  /* 'C:/' */
-		DP(o)->driver = konoha_getIODriver(ctx, STEXT("file"));
+		DP(o)->driver = knh_getIODriver(ctx, STEXT("file"));
 	}
 	else {
-		DP(o)->driver = konoha_getIODriver(ctx, knh_bytes_first(fname, loc));
+		DP(o)->driver = knh_getIODriver(ctx, knh_bytes_first(fname, loc));
 	}
 	KNH_SETv(ctx, DP(o)->urn, new_String(ctx, fname, NULL));
 	char *mode = "r";
-	if(IS_NOTNULL(m)) mode = knh_String_tochar(m);
-	DP(o)->fd = DP(o)->driver->fopen(ctx, fname, mode);
+	if(IS_NOTNULL(m)) mode = __tochar(m);
+	DP(o)->fd = DP(o)->driver->fopen(ctx, fname, mode, knh_Context_isStrict(ctx));
 	if(DP(o)->fd != -1) {
 		KNH_SETv(ctx, DP(o)->urn, urn);
 		DP(o)->bufsiz = DP(o)->driver->bufsiz;
@@ -105,14 +106,14 @@ Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn, String *m)
 		DP(o)->driver->finit(ctx, (Object*)o, mode);
 	}
 	else {
-		DP(o)->driver = konoha_getDefaultIODriver();
+		DP(o)->driver = knh_getDefaultIODriver();
 	}
 	return (Object*)o;
 }
 
 /* ------------------------------------------------------------------------ */
 
-int knh_InputStream_getc(Ctx *ctx, InputStream *o)
+int knh_InputStream_getc(Ctx *ctx, knh_InputStream_t *o)
 {
 	int ch;
 	if(knh_InputStream_isFILE(o)) {
@@ -139,7 +140,6 @@ int knh_InputStream_getc(Ctx *ctx, InputStream *o)
 		}else if(ch == '\r') {
 			DP(o)->line++;
 		}
-		DP(o)->prev = ch;
 	}
 	return ch;
 }
@@ -147,7 +147,7 @@ int knh_InputStream_getc(Ctx *ctx, InputStream *o)
 /* ------------------------------------------------------------------------ */
 
 size_t
-knh_InputStream_read(Ctx *ctx, InputStream *o, char *buf, size_t bufsiz)
+knh_InputStream_read(Ctx *ctx, knh_InputStream_t *o, char *buf, size_t bufsiz)
 {
 	if(knh_InputStream_isFILE(o)) {
 		return knh_fread(ctx, buf, bufsiz, DP(o)->fp);
@@ -162,7 +162,9 @@ knh_InputStream_read(Ctx *ctx, InputStream *o, char *buf, size_t bufsiz)
 			}
 			return bufsiz;
 		}
-		knh_memcpy(buf, DP(o)->buf, inbufsiz);
+		// XXX when both InputStream.read and InputStream.readLine method call,
+		// it seams strange. so, move DP(o)->buf's pointer to bufpos.
+		knh_memcpy(buf, DP(o)->buf + DP(o)->bufpos, inbufsiz);
 		DP(o)->bufpos += inbufsiz;
 		DP(o)->size += bufsiz;
 		buf += inbufsiz;
@@ -173,62 +175,90 @@ knh_InputStream_read(Ctx *ctx, InputStream *o, char *buf, size_t bufsiz)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method String InputStream.readLine() */
 
-String* knh_InputStream_readLine(Ctx *ctx, InputStream *o)
+knh_String_t* knh_InputStream_readLine(Ctx *ctx, knh_InputStream_t *in)
 {
 	int ch;
-	knh_cwb_t cwb = new_cwb(ctx);
-	while((ch = knh_InputStream_getc(ctx, o)) != EOF) {
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+	while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
 		if(ch == '\r') {
-			return new_String__cwbconv(ctx, cwb, DP(o)->bconv);
+			DP(in)->prev = ch;
+			return new_String__cwbconv(ctx, cwb, DP(in)->bconv);
 		}
-		if(ch == '\n') {
-			if(DP(o)->prev == '\r') continue;
-			return new_String__cwbconv(ctx, cwb, DP(o)->bconv);
+		else if(ch == '\n') {
+			if(DP(in)->prev == '\r') continue;
+			DP(in)->prev = ch;
+			return new_String__cwbconv(ctx, cwb, DP(in)->bconv);
+		}
+		else {
+			knh_Bytes_putc(ctx, cwb->ba, ch);
 		}
-		knh_Bytes_putc(ctx, cwb.ba, ch);
 	}
 	if(knh_cwb_size(cwb) != 0) {
-		return new_String__cwbconv(ctx, cwb, DP(o)->bconv);
+		return new_String__cwbconv(ctx, cwb, DP(in)->bconv);
 	}
-	return (String*)KNH_NULL;
+	return (knh_String_t*)KNH_NULL;
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void InputStream.close() */
 
-void knh_InputStream_close(Ctx *ctx, InputStream *o)
+void knh_InputStream_close(Ctx *ctx, knh_InputStream_t *o)
 {
 	f_io_close f = DP(o)->driver->fclose;
-	DP(o)->driver = konoha_getDefaultIODriver();
+	DP(o)->driver = knh_getDefaultIODriver();
 	f(ctx, DP(o)->fd);
 	DP(o)->fd = -1;
+	KNH_SETv(ctx, DP(o)->ba, KNH_NULL);
 }
 
 /* ------------------------------------------------------------------------ */
 
-int knh_InputStream_isClosed(InputStream *o)
+int knh_InputStream_isClosed(Ctx *ctx, knh_InputStream_t *o)
 {
-	return (DP(o)->fd == -1);
+	return (DP(o)->fd == -1 && IS_NULL(DP(o)->ba));
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_InputStream_setEncoding(Ctx *ctx, knh_InputStream_t *o, knh_String_t *enc)
+{
+	if(IS_NULL(enc)) {
+		KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
+		KNH_SETv(ctx, DP(o)->bconv, KNH_NULL);
+	}
+	else if(knh_bytes_strcasecmp(__tobytes(enc), STEXT(KONOHA_ENCODING)) == 0) {
+		KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
+		KNH_SETv(ctx, DP(o)->bconv, KNH_NULL);
+	}
+	else {
+		knh_BytesConv_t *bin = new_BytesConv__in(ctx, __tochar(enc));
+		if(IS_NULL(bin)) {
+			KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
+			KNH_WARNING(ctx, "unsupported character encoding: %s", __tochar(enc));
+		}
+		else {
+			KNH_SETv(ctx, DP(o)->enc, enc);
+		}
+		KNH_SETv(ctx, DP(o)->bconv, bin);
+	}
 }
 
 /* ======================================================================== */
 /* [File] */
 
-KNHAPI(InputStream*) new_FileInputStream(Ctx *ctx, knh_bytes_t file)
+KNHAPI(knh_InputStream_t*) new_FileInputStream(Ctx *ctx, knh_bytes_t file, int isThrowable)
 {
-	knh_iodrv_t *drv = konoha_getIODriver(ctx, STEXT("file"));
-	knh_io_t fd = drv->fopen(ctx, file, "r");
+	knh_iodrv_t *drv = knh_getIODriver(ctx, STEXT("file"));
+	knh_io_t fd = drv->fopen(ctx, file, "r", isThrowable);
 	return new_InputStream__io(ctx, new_String(ctx, file, NULL), fd, drv);
 }
 
 /* ======================================================================== */
 /* [Bytes] */
 
-KNHAPI(InputStream*) new_BytesInputStream(Ctx *ctx, Bytes *ba, size_t s, size_t e)
+KNHAPI(knh_InputStream_t*) new_BytesInputStream(Ctx *ctx, knh_Bytes_t *ba, size_t s, size_t e)
 {
-	InputStream* o = (InputStream*)new_Object_bcid(ctx, CLASS_InputStream, 0);
+	knh_InputStream_t* o = (knh_InputStream_t*)new_Object_bcid(ctx, CLASS_InputStream, 0);
 	DP(o)->fd = -1;
 	KNH_ASSERT(IS_Bytes(ba));
 	KNH_SETv(ctx, DP(o)->ba, ba);
@@ -246,23 +276,23 @@ KNHAPI(InputStream*) new_BytesInputStream(Ctx *ctx, Bytes *ba, size_t s, size_t
 
 MAPPER knh_Bytes_InputStream(Ctx *ctx, knh_sfp_t *sfp)
 {
-	Bytes *ba = (Bytes*)sfp[0].o;
+	knh_Bytes_t *ba = (knh_Bytes_t*)sfp[0].o;
 	KNH_MAPPED(ctx, sfp, new_BytesInputStream(ctx, ba, 0, knh_Bytes_size(ba)));
 }
 
 /* ======================================================================== */
-/* [Bytes] */
+/* [String] */
 
-KNHAPI(InputStream*) new_StringInputStream(Ctx *ctx, String *str, size_t s, size_t e)
+KNHAPI(knh_InputStream_t*) new_StringInputStream(Ctx *ctx, knh_String_t *str, size_t s, size_t e)
 {
-	InputStream* o = (InputStream*)new_Object_bcid(ctx, CLASS_InputStream, 0);
+	knh_InputStream_t* o = (knh_InputStream_t*)new_Object_bcid(ctx, CLASS_InputStream, 0);
 	DP(o)->fd = -1;
-	KNH_ASSERT(IS_Bytes(str));
+	DBG2_ASSERT(IS_Bytes(str));
 	KNH_SETv(ctx, DP(o)->str, str);
 	DP(o)->buf = (char*)(str)->str;
 	DP(o)->bufsiz = (str)->size;
-	KNH_ASSERT(e <= (str)->size);
-	KNH_ASSERT(s <= e);
+	DBG2_ASSERT(e <= (str)->size);
+	DBG2_ASSERT(s <= e);
 	DP(o)->bufpos   = s;
 	DP(o)->bufend   = e;
 	return o;
