@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -29,13 +29,11 @@
 
 #include"commons.h"
 
-#ifdef KNH_USING_REGEX
-#ifdef KONOHA_OS__WINDOWS
+#if defined(KNH_USING_REGEX) && defined(KNH_USING_ONIGURUMA)
 #include "onigposix.h"
-#else
+#elif defined(KNH_USING_REGEX)
 #include<regex.h>
 #endif
-#endif
 
 /* ************************************************************************ */
 
@@ -85,10 +83,10 @@ void knh_regex_regfree__NOP(Ctx *ctx, knh_regex_t *reg)
 
 /* ======================================================================== */
 
-void knh_write_USING_REGEX(Ctx *ctx, OutputStream *w)
+void knh_write_USING_REGEX(Ctx *ctx, knh_OutputStream_t *w)
 {
 #ifdef KNH_USING_REGEX
-	knh_write__s(ctx, w, " regex");
+	knh_write_char(ctx, w, " regex");
 #endif
 }
 
@@ -113,7 +111,7 @@ int knh_regex_regcomp(Ctx *ctx, knh_regex_t *reg, char *pattern, char *option)
 	}
 	int res = regcomp((regex_t*)reg, pattern, flag);
 	if(res != 0) {
-		char buf[FILENAME_BUFSIZ * 4];
+		char buf[FILEPATH_BUFSIZ * 4];
 		regerror(res, (regex_t*)reg, buf, sizeof(buf));
 		KNH_WARNING(ctx, "regex(error): %s", buf);
 	}
@@ -181,10 +179,10 @@ static knh_regex_drvapi_t RE__regex = {
 /* [string parser] */
 
 static
-Regex *new_Regex(Ctx *ctx, String *pattern, String *option)
+knh_Regex_t *new_Regex(Ctx *ctx, knh_String_t *pattern, knh_String_t *option)
 {
 	knh_Regex_t *o = (knh_Regex_t*)new_hObject(ctx, FLAG_Regex, CLASS_Regex, CLASS_Regex);
-	knh_bytes_t p = knh_String_tobytes(pattern);
+	knh_bytes_t p = __tobytes(pattern);
 	knh_index_t loc = knh_bytes_index(p, ':');
 	KNH_INITv(o->pattern, pattern);
 	if(loc == -1) {
@@ -196,7 +194,7 @@ Regex *new_Regex(Ctx *ctx, String *pattern, String *option)
 	o->reg = o->df->regmalloc(ctx);
 	{
 		char *ptn = (char*)(knh_bytes_last(p, loc+1).buf);
-		char *opt = IS_NULL(option) ? "" : knh_String_tochar(option);
+		char *opt = IS_NULL(option) ? "" : __tochar(option);
 		o->df->regcomp(ctx, o->reg, ptn, opt);
 	}
 	return o;
@@ -205,9 +203,9 @@ Regex *new_Regex(Ctx *ctx, String *pattern, String *option)
 /* ------------------------------------------------------------------------ */
 
 static
-Object* new_Regex_parseOf(Ctx *ctx, String *pattern)
+Object* new_Regex_parseOf(Ctx *ctx, knh_String_t *pattern)
 {
-	return (Object*)new_Regex(ctx, pattern, (String*)KNH_NULL);
+	return (Object*)new_Regex(ctx, pattern, (knh_String_t*)KNH_NULL);
 }
 
 static knh_parser_drvapi_t PARSER__Regex = {
@@ -220,7 +218,7 @@ static knh_parser_drvapi_t PARSER__Regex = {
 
 knh_regex_drvapi_t *knh_System_getRegexDriver(Ctx *ctx, knh_bytes_t name)
 {
-	knh_regex_drvapi_t *p = (knh_regex_drvapi_t *)konoha_getDriverAPI(ctx, KNH_DRVAPI_TYPE__REGEX, name);
+	knh_regex_drvapi_t *p = (knh_regex_drvapi_t *)knh_getDriverAPI(ctx, KNH_DRVAPI_TYPE__REGEX, name);
 	if(p == NULL) {
 		KNH_WARNING(ctx, "Regex: unsupported scheme '%s'", name);
 		p = &RE__NOP;
@@ -230,27 +228,27 @@ knh_regex_drvapi_t *knh_System_getRegexDriver(Ctx *ctx, knh_bytes_t name)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) konoha_addRegexDriver(Ctx *ctx, char *alias, knh_regex_drvapi_t *d)
+KNHAPI(void) knh_addRegexDriver(Ctx *ctx, char *alias, knh_regex_drvapi_t *d)
 {
 	if(alias != NULL) {
-		konoha_addParserDriver(ctx, alias, &PARSER__Regex);
+		knh_addParserDriver(ctx, alias, &PARSER__Regex);
 	}
 	else {
-		konoha_addParserDriver(ctx, d->name, &PARSER__Regex);
+		knh_addParserDriver(ctx, d->name, &PARSER__Regex);
 	}
-	konoha_addDriverAPI(ctx, alias, (knh_drvapi_t*)d);
+	knh_addDriverAPI(ctx, alias, (knh_drvapi_t*)d);
 }
 
 /* ======================================================================== */
 /* [init] */
 
-void KNHINIT init_Regex(Ctx *ctx)
+void knh_Regex_newClass(Ctx *ctx, knh_class_t cid)
 {
-	konoha_addRegexDriver(ctx, NULL, &RE__NOP);
+	knh_addRegexDriver(ctx, NULL, &RE__NOP);
 #ifdef KNH_USING_REGEX
-	konoha_addRegexDriver(ctx, NULL, &RE__regex);
-	konoha_addRegexDriver(ctx, "re", &RE__regex);
-	konoha_setSystemPropertyText(ctx, "konoha.regex", "regex/POSIX.2");
+	knh_addRegexDriver(ctx, NULL, &RE__regex);
+	knh_addRegexDriver(ctx, "re", &RE__regex);
+	knh_setSystemPropertyText(ctx, "konoha.regex", "POSIX.2");
 #endif
 }
 
