@@ -43,10 +43,10 @@ extern "C" {
 /* ======================================================================== */
 /* [ctxkey] */
 
-static int isInit = 0;
 
 #ifdef KNH_USING_THREAD
 static knh_thread_key_t ctxkey;
+static Ctx *curctx = NULL;
 #else
 static Ctx *curctx = NULL;
 #endif
@@ -55,9 +55,9 @@ static Ctx *curctx = NULL;
 
 void konoha_init(void)
 {
+	static int isInit = 0;
 #ifdef KNH_USING_THREAD
 	if(isInit == 0) {
-		isInit = 1;
 		knh_thread_key_create(&ctxkey);
 	}
 #endif
@@ -73,9 +73,8 @@ KNHAPI(Ctx*) knh_beginContext(Ctx *ctx)
 #ifdef KNH_USING_THREAD
 	knh_mutex_lock(ctx->ctxlock);
 	knh_thread_setspecific(ctxkey, ctx);
-#else
-	curctx = ctx;
 #endif
+	curctx = ctx;
 	return ctx;
 }
 
@@ -86,9 +85,8 @@ KNHAPI(void) knh_endContext(Ctx *ctx)
 #ifdef KNH_USING_THREAD
 	knh_thread_setspecific(ctxkey, NULL);
 	knh_mutex_unlock(ctx->ctxlock);
-#else
-	curctx = NULL;
 #endif
+	curctx = NULL;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -98,6 +96,9 @@ KNHAPI(Ctx*) knh_getCurrentContext(void)
 	Ctx *ctx;
 #ifdef KNH_USING_THREAD
 	ctx = (Ctx*)knh_thread_getspecific(ctxkey);
+	if (ctx == NULL) {
+		ctx = (Ctx *) curctx;
+	}
 #else
 	ctx = curctx;
 #endif
@@ -678,7 +679,7 @@ static void knh_initSIGINT(void)
 
 #if defined(KNH_USING_READLINE)
 
-volatile static Ctx *ctxRL = NULL;
+static volatile Ctx *ctxRL = NULL;
 
 /* ------------------------------------------------------------------------ */
 
@@ -729,7 +730,7 @@ char *knh_rl_stmt(const char *text, int state)
 static
 knh_bytes_t knh_bytes_token(knh_bytes_t t)
 {
-	int i;
+	size_t i;
 	for(i = 0; i < t.len; i++) {
 		if(isalnum(t.buf[i])) {
 			t.buf = t.buf + i + 1;
@@ -745,7 +746,7 @@ knh_bytes_t knh_bytes_token(knh_bytes_t t)
 static
 char *knh_rl_name(const char *text, int state)
 {
-	static int index;
+	static size_t index;
 	static knh_bytes_t t = {(knh_uchar_t*)"", 0};
 	Ctx *ctx = (Ctx*)ctxRL;
 	if(state == 0) {
@@ -771,7 +772,7 @@ char *knh_rl_name(const char *text, int state)
 static void knh_Context_initSymbolTable(Ctx *ctx)
 {
 	knh_DictMap_t *symbolDictMap = DP((ctx)->kc)->symbolDictMap;
-	int i, j;
+	size_t i, j;
 	for(i = 0; i < KNH_TCLASS_SIZE; i++) {
 		knh_String_t *sname = ClassTable(i).sname;
 		if(sname != NULL) {
@@ -887,7 +888,7 @@ static
 void knh_clearScriptLine(Ctx *ctx)
 {
 	if(ctx->lines != NULL) {
-#if KONOHA_BUILDID > KONOHA_BUILDID_LATEST
+//#if KONOHA_BUILDID > KONOHA_BUILDID_LATEST
 //		if(knh_getenv("I_LOVE_KONOHA") == NULL) {
 //			char buff[FILEPATH_BUFSIZ];
 //			int i, pid = knh_getpid();
@@ -915,7 +916,7 @@ void knh_clearScriptLine(Ctx *ctx)
 //				fprintf(stdout, "Thank you, again.\n");
 //			}
 //		}
-#endif
+//#endif
 		KNH_FINALv(ctx, ((knh_Context_t*)ctx)->lines);
 		((knh_Context_t*)ctx)->lines = NULL;
 	}
@@ -927,8 +928,8 @@ void knh_clearScriptLine(Ctx *ctx)
 static
 void knh_showWelcome(Ctx *ctx, knh_OutputStream_t *w)
 {
-	knh_printf(ctx, w, "Konoha %s(%s) %s (rev:%d, %s %s)",
-		KONOHA_VERSION, KONOHA_XCODE, KONOHA_LICENSE,
+	knh_printf(ctx, w, "Konoha %s(%s) %s/%s (rev:%d, %s %s)",
+		KONOHA_VERSION, KONOHA_XCODE, KONOHA_DIST, KONOHA_LICENSE,
 		((size_t)KONOHA_REVISION), __DATE__, __TIME__);
 	knh_write_EOL(ctx, w);
 	knh_printf(ctx, w, "[%s] on %s (%d, %s)\n", KONOHA_CC_VERSION, KONOHA_PLATFORM, (knh_intptr_t)(sizeof(void*) * 8), konoha_encoding());
@@ -981,6 +982,7 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 	using_history();
 #endif
 	knh_showWelcome(ctx, KNH_STDOUT);
+	knh_check_update(ctx);
 	knh_initScriptLine(ctx);
 	{
 		int linenum, linecnt = 0;
