@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -28,10 +28,17 @@
 /* ************************************************************************ */
 
 #include"commons.h"
+
+#if defined(KONOHA_ON_LKM)
+#if defined(__linux__)
+#include <linux/random.h>
+#endif
+#else
 #include<time.h>
 #ifdef KNH_USING_POSIX
 #include<unistd.h>
 #endif
+#endif
 
 /* ************************************************************************ */
 
@@ -46,47 +53,68 @@ void init_genrand(unsigned long s);
 void init_genrand64(unsigned long long seed);
 unsigned long genrand_int31(void);
 unsigned long long genrand64_int63(void);
+#ifndef KONOHA_ON_LKM
 double genrand_real1(void);
 double genrand64_real1(void);
+#endif
 
 /* ------------------------------------------------------------------------ */
 
 void knh_srand(knh_uint_t seed)
 {
 	if(seed == 0) {
+#ifdef KONOHA_ON_LKM
+		seed = 1;
+#else
 #ifdef KNH_USING_POSIX
 		seed = (knh_uint_t)time(NULL) + getpid();
 #else
 		seed = (knh_uint_t)time(NULL);
+#endif
 #endif
 	}
+#ifndef KONOHA_ON_LKM
 #ifdef KNH_USING_INT32
 	init_genrand((unsigned long)seed);
 #else
 	init_genrand64((unsigned long long int)seed);
 #endif
+#endif /* KONOHA_ON_LKM */
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_uint_t knh_rand()
+knh_uint_t knh_rand(void)
 {
+#ifdef KONOHA_ON_LKM
+#if defined(__linux__)
+	return (knh_uint_t)random32();
+#else
+	return (knh_uint_t)random();
+#endif
+#else
 #ifdef KNH_USING_INT32
 	return (knh_uint_t)genrand_int31();
 #else
 	return (knh_uint_t)genrand64_int63();
 #endif
+#endif
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_float_t knh_float_rand()
+knh_float_t knh_float_rand(void)
 {
+#ifdef KNH_USING_NOFLOAT
+	return (knh_float_t)knh_rand();
+#else
+
 #ifdef KNH_USING_INT32
 	return (knh_float_t)genrand_real1();
 #else
 	return (knh_float_t)genrand64_real1();
 #endif
+#endif /* KNH_USING_NOFLOAT */
 }
 
 /* ------------------------------------------------------------------------ */
