@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -57,13 +57,12 @@ static int canFree = 1;
 
 void *knh_malloc(Ctx *ctx, size_t size)
 {
-#ifdef KONOHA_OS__TB
-	if(size == 0) size=4;
-#endif
+	KNH_ASSERT(size>0);
 	void *block = malloc(size);
 	if (unlikely(block == NULL)) {
+		DBG2_ABORT();
 		KNH_EXIT("OutOfMemory!!: %d bytes used", (int)ctx->stat->usedMemorySize);
-		//KNH_THROWs(ctx, "OutOfMemory!!");
+		//KNH_THROW__T(ctx, "OutOfMemory!!");
 	}
 	knh_stat_incUsedMemorySize(ctx, size);
 	return block;
@@ -86,8 +85,9 @@ void *DBG2_malloc(Ctx *ctx, size_t size, char *func)
 	size_t *block = (size_t*)malloc(size + sizeof(size_t));
 	//if(size >32) { fprintf(stdout, "%p: M(%s, size=%d)\n", block, func, (int)size); };
 	if (unlikely(block == NULL)) {
+		DBG2_ABORT();
 		KNH_EXIT("OutOfMemory!!: %d bytes used", (int)ctx->stat->usedMemorySize);
-		//KNH_THROWs(ctx, "OutOfMemory!!");
+		//KNH_THROW__T(ctx, "OutOfMemory!!");
 	}
 	knh_stat_incUsedMemorySize(ctx, size);
 	block[0] = size;
@@ -100,7 +100,7 @@ void DBG2_free(Ctx *ctx, void *p, size_t size, char *func)
 {
 	DBG_ASSERT_FREE();
 	size_t *block = ((size_t*)p) - 1;
-	if(size != block[0]) {
+	if(unlikely(size != block[0])) {
 		fprintf(stderr, "%s: ptr = %p, block.size = %d, free.size=%d", func, p, (int)block[0], (int)size);
 		KNH_ASSERT(size == block[0]);
 	}
@@ -120,13 +120,6 @@ void DBG2_free(Ctx *ctx, void *p, size_t size, char *func)
 	}\
 
 
-///* ------------------------------------------------------------------------ */
-//
-//static void* knh_pagetop(void *p)
-//{
-//	return (void*)((((knh_uint_t)p) / KONOHA_PAGESIZE) * KONOHA_PAGESIZE);
-//}
-
 /* ------------------------------------------------------------------------ */
 
 static
@@ -141,10 +134,10 @@ knh_Object_t *new_UnusedObject(Ctx *ctx)
 		knh_bzero(newpage, sizeof(knh_ObjectPageTable_t) * newsize);
 		knh_memcpy(newpage, ctx->share->ObjectPageTable, ctx->share->ObjectPageTableMaxSize);
 		KNH_FREE(ctx, ctx->share->ObjectPageTable, ctx->share->ObjectPageTableMaxSize);
-		ctx->share->ObjectPageTable = newpage;
-		ctx->share->ObjectPageTableMaxSize = newsize;
+		((knh_SharedData_t*)ctx->share)->ObjectPageTable = newpage;
+		((knh_SharedData_t*)ctx->share)->ObjectPageTableMaxSize = newsize;
 	}
-	ctx->share->ObjectPageTableSize += 1;
+	((knh_SharedData_t*)ctx->share)->ObjectPageTableSize += 1;
 	KNH_UNLOCK(ctx, LOCK_MEMORY, NULL);
 
 	char *t = (char*)KNH_MALLOC(ctx, SIZEOF_OBJECTPAGE);
@@ -152,11 +145,14 @@ knh_Object_t *new_UnusedObject(Ctx *ctx)
 	ctx->share->ObjectPageTable[tindex].ctxid = ctx->ctxid;
 	ctx->share->ObjectPageTable[tindex].thead = t;
 	if((knh_uintptr_t)t % KONOHA_PAGESIZE != 0) {
-		t = (char*)((((knh_uintptr_t)t / KONOHA_PAGESIZE) + 1) * KONOHA_PAGESIZE);
+		char *t2 = (char*)((((knh_uintptr_t)t / KONOHA_PAGESIZE) + 1) * KONOHA_PAGESIZE);
+		DBG_P("KONOHA_PAGESIZE=%d, shift=%d", (int)KONOHA_PAGESIZE, (int)(t2 - t));
+		t = t2;
 		KNH_ASSERT((knh_uintptr_t)t % KONOHA_PAGESIZE == 0);
 	}
 	char *h = t, *max = ctx->share->ObjectPageTable[tindex].thead + SIZEOF_OBJECTPAGE;
 	int cnt = 0;
+	//DBG_P("OBJECTS IN PAGESIZE=%d", KONOHA_PAGESIZE / sizeof(knh_Object_t));
 	while(t + KONOHA_PAGESIZE < max) {
 		size_t i;
 		knh_Object_t *o = (knh_Object_t*)t;
@@ -174,8 +170,8 @@ knh_Object_t *new_UnusedObject(Ctx *ctx)
 		}
 		cnt++;
 	}
-	((Context*)ctx)->unusedObject = ((knh_Object_t*)h) + 1;
-	((Context*)ctx)->unusedObjectSize += cnt;
+	((knh_Context_t*)ctx)->unusedObject = ((knh_Object_t*)h) + 1;
+	((knh_Context_t*)ctx)->unusedObjectSize += cnt;
 	DBG_P("*** new object page tindex=%d, ctxid=%d unused=%p,%d ***",
 			tindex, ctx->ctxid, ctx->unusedObject, (int)ctx->unusedObjectSize);
 	return ctx->unusedObject;
@@ -184,14 +180,12 @@ knh_Object_t *new_UnusedObject(Ctx *ctx)
 /* ------------------------------------------------------------------------ */
 /* [fastmalloc] */
 
-/* ------------------------------------------------------------------------ */
-
 static void knh_setFastMallocMemory(void *p)
 {
 	knh_uintptr_t *b = (knh_uintptr_t*)((((knh_uintptr_t)p) / KONOHA_PAGESIZE) * KONOHA_PAGESIZE);
-	int n = ((knh_uintptr_t)p % KONOHA_PAGESIZE) / sizeof(knh_Object_t);
-	int x = n/(sizeof(knh_uintptr_t)*8);
-	knh_uintptr_t mask = 1 << (n % (sizeof(knh_uintptr_t)*8));
+	int n = ((Object*)p - (Object*)b); /*((knh_uintptr_t)p % KONOHA_PAGESIZE) / sizeof(knh_Object_t); */
+	int x = n / (sizeof(knh_uintptr_t)*8);
+	knh_uintptr_t mask = ((knh_uintptr_t)1) << (n % (sizeof(knh_uintptr_t)*8));
 	b[x] = b[x] | mask;
 }
 
@@ -200,9 +194,9 @@ static void knh_setFastMallocMemory(void *p)
 static void knh_unsetFastMallocMemory(void *p)
 {
 	knh_uintptr_t *b = (knh_uintptr_t*)((((knh_uintptr_t)p) / KONOHA_PAGESIZE) * KONOHA_PAGESIZE);
-	int n = ((knh_uintptr_t)p % KONOHA_PAGESIZE) / sizeof(knh_Object_t);
-	int x = n/(sizeof(knh_uintptr_t)*8);
-	knh_uintptr_t mask = 1 << (n % (sizeof(knh_uintptr_t)*8));
+	int n = ((Object*)p - (Object*)b); /*((knh_uintptr_t)p % KONOHA_PAGESIZE) / sizeof(knh_Object_t); */
+	int x = n / (sizeof(knh_uintptr_t)*8);
+	knh_uintptr_t mask = ((knh_uintptr_t)1) << (n % (sizeof(knh_uintptr_t)*8));
 	b[x] = b[x] & ~(mask);
 }
 
@@ -211,21 +205,26 @@ static void knh_unsetFastMallocMemory(void *p)
 int knh_isFastMallocMemory(void *p)
 {
 	knh_uintptr_t *b = (knh_uintptr_t*)((((knh_uintptr_t)p) / KONOHA_PAGESIZE) * KONOHA_PAGESIZE);
-	int n = ((knh_uintptr_t)p % KONOHA_PAGESIZE) / sizeof(knh_Object_t);
-	int x = n/(sizeof(knh_uintptr_t)*8);
-	knh_uintptr_t mask = 1 << (n % (sizeof(knh_uintptr_t)*8));
+	int n = ((Object*)p - (Object*)b); /*((knh_uintptr_t)p % KONOHA_PAGESIZE) / sizeof(knh_Object_t); */
+	int x = n / (sizeof(knh_uintptr_t)*8);
+	knh_uintptr_t mask = ((knh_uintptr_t)1) << (n % (sizeof(knh_uintptr_t)*8));
 	return ((b[x] & mask) == mask);
 }
 
 /* ------------------------------------------------------------------------ */
+#define CHECK_UNUSED_OBJECT(ctx) { \
+if(unlikely(ctx->unusedObject == NULL)) { \
+		KNH_ASSERT(ctx->unusedObjectSize == 0); \
+		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx); \
+	} \
+} \
+/* ------------------------------------------------------------------------ */
+
 
 void *knh_fastmalloc(Ctx *ctx, size_t size)
 {
 	if(size <= KNH_FASTMALLOC_SIZE) {
-		if(unlikely(ctx->unusedObject == NULL)) {
-			KNH_ASSERT(ctx->unusedObjectSize == 0);
-			((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
-		}
+		CHECK_UNUSED_OBJECT(ctx);
 		Object *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
 		((knh_Context_t*)ctx)->unusedObjectSize -= 1;
@@ -233,13 +232,16 @@ void *knh_fastmalloc(Ctx *ctx, size_t size)
 		knh_setFastMallocMemory((void*)o);
 		return (void*)o;
 	}
-	void *block = malloc(size);
-	if (unlikely(block == NULL)) {
-		KNH_EXIT("OutOfMemory!!: %d bytes used", (int)ctx->stat->usedMemorySize);
-		//KNH_THROWs(ctx, "OutOfMemory!!");
+	else {
+		void *block = malloc(size);
+		if (unlikely(block == NULL)) {
+			DBG2_ABORT();
+			KNH_EXIT("OutOfMemory!!: %d bytes used", (int)ctx->stat->usedMemorySize);
+			//KNH_THROW__T(ctx, "OutOfMemory!!");
+		}
+		knh_stat_incUsedMemorySize(ctx, size);
+		return block;
 	}
-	knh_stat_incUsedMemorySize(ctx, size);
-	return block;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -271,10 +273,7 @@ knh_Object_t *new_hObject(Ctx *ctx, knh_flag_t flag, knh_class_t bcid, knh_class
 	DBG2_ASSERT(bcid != CLASS_Context);
 	DBG2_ASSERT(bcid < KNH_TSTRUCT_SIZE);
 	DBG2_ASSERT(ctx != NULL);
-	if(unlikely(ctx->unusedObject == NULL)) {
-		KNH_ASSERT(ctx->unusedObjectSize == 0);
-		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
-	}
+	CHECK_UNUSED_OBJECT(ctx);
 	{
 		knh_Object_t *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
@@ -286,7 +285,7 @@ knh_Object_t *new_hObject(Ctx *ctx, knh_flag_t flag, knh_class_t bcid, knh_class
 		o->h.bcid = bcid;
 		o->h.cid  = cid;
 		o->h.ctxid = ctx->ctxid;
-		o->h.lock  = LOCK_NO;
+		o->h.lock  = LOCK_NOP;
 		DBG2_({o->ref = NULL;});
 		return o;
 	}
@@ -298,10 +297,7 @@ knh_Object_t *new_Object_bcid(Ctx *ctx, knh_class_t bcid, int init)
 {
 	DBG2_ASSERT(bcid != CLASS_Context);
 	DBG2_ASSERT(bcid < KNH_TSTRUCT_SIZE);
-	if(unlikely(ctx->unusedObject == NULL)) {
-		KNH_ASSERT(ctx->unusedObjectSize == 0);
-		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
-	}
+	CHECK_UNUSED_OBJECT(ctx);
 	{
 		knh_Object_t *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
@@ -313,7 +309,7 @@ knh_Object_t *new_Object_bcid(Ctx *ctx, knh_class_t bcid, int init)
 		o->h.bcid = bcid;
 		o->h.cid  = bcid;
 		o->h.ctxid = ctx->ctxid;
-		o->h.lock  = LOCK_NO;
+		o->h.lock  = LOCK_NOP;
 		size_t size = ctx->share->StructTable[bcid].size;
 		//DBG2_P("cid=%d,%s,size=%d", bcid, STRUCTN(bcid), size);
 		if(size > 0) {
@@ -332,10 +328,7 @@ knh_Object_t *new_Object_bcid(Ctx *ctx, knh_class_t bcid, int init)
 knh_Object_t *new_Object_init(Ctx *ctx, knh_flag_t flag, knh_class_t cid, int init)
 {
 	DBG2_ASSERT(cid != CLASS_Context);
-	if(ctx->unusedObject == NULL) {
-		KNH_ASSERT(ctx->unusedObjectSize == 0);
-		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
-	}
+	CHECK_UNUSED_OBJECT(ctx);
 	{
 		knh_Object_t *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
@@ -343,14 +336,14 @@ knh_Object_t *new_Object_init(Ctx *ctx, knh_flag_t flag, knh_class_t cid, int in
 		knh_stat_incUsedObjectSize(ctx, 1);
 		o->h.magic = KNH_OBJECT_MAGIC;
 		knh_Object_RCset(o, KNH_RCGC_INIT);
-		KNH_ASSERT_cid(cid);
-		knh_class_t bcid = ctx->share->ClassTable[cid].bcid;
+		DBG2_ASSERT_cid(cid);
+		knh_class_t bcid = ClassTable(cid).bcid;
 		o->h.bcid = bcid;
-		o->h.flag = ctx->share->ClassTable[cid].oflag | flag;
+		o->h.flag = ClassTable(cid).oflag | flag;
 		o->h.cid  = cid;
 		o->h.ctxid = ctx->ctxid;
-		o->h.lock  = LOCK_NO;
-		size_t size = ctx->share->ClassTable[cid].size;
+		o->h.lock  = LOCK_NOP;
+		size_t size = ClassTable(cid).size;
 		if(size > 0) {
 			o->ref = KNH_MALLOC(ctx, size);
 		}
@@ -367,10 +360,7 @@ knh_Object_t *new_Object_init(Ctx *ctx, knh_flag_t flag, knh_class_t cid, int in
 KNHAPI(Object*) new_Object_boxing(Ctx *ctx, knh_class_t cid, knh_sfp_t *sfp)
 {
 	KNH_ASSERT(ctx != NULL);
-	if(unlikely(ctx->unusedObject == NULL)) {
-		KNH_ASSERT(ctx->unusedObjectSize == 0);
-		((knh_Context_t*)ctx)->unusedObject = new_UnusedObject(ctx);
-	}
+	CHECK_UNUSED_OBJECT(ctx);
 	{
 		knh_Object_t *o = ctx->unusedObject;
 		((knh_Context_t*)ctx)->unusedObject = (knh_Object_t*)o->ref;
@@ -379,10 +369,10 @@ KNHAPI(Object*) new_Object_boxing(Ctx *ctx, knh_class_t cid, knh_sfp_t *sfp)
 		o->h.magic = KNH_OBJECT_MAGIC;
 		knh_Object_RCset(o, KNH_RCGC_INIT);
 		o->h.flag = FLAG_Float;
-		o->h.bcid = ctx->share->ClassTable[cid].bcid;
+		o->h.bcid = ClassTable(cid).bcid;
 		o->h.cid  = cid;
 		o->h.ctxid = ctx->ctxid;
-		o->h.lock  = LOCK_NO;
+		o->h.lock  = LOCK_NOP;
 		knh_Float_t *v = (knh_Float_t*)o;
 		v->n.data = sfp[0].data;
 		return o;
@@ -395,23 +385,17 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 {
 	DBG_ASSERT_FREE();
 #ifdef KNH_USING_RCGC
-	KNH_ASSERT(knh_Object_isRC0(o));
+	DBG2_ASSERT(knh_Object_isRC0(o));
 #endif
 //	DBG2_P("o=%p, sid=%d,%s", o, o->h.bcid, STRUCTN(o->h.bcid));
 	if(unlikely(o->h.magic == 0)) return;
+	if(unlikely(o->h.bcid == CLASS_Context)) return;
 	o->h.magic = 0;
-	ctx->share->StructTable[o->h.bcid].ftraverse(ctx, o, knh_Object_sweep);
-	size_t size = ctx->share->ClassTable[o->h.cid].size;
+	StructTable(o->h.bcid).ftraverse(ctx, o, knh_Object_sweep);
+	size_t size = ClassTable(o->h.cid).size;
 	if(size > 0) {
 		KNH_FREE(ctx, o->ref, size);
 	}
-#ifdef KNH_FLAG_OF_METADATA
-	if((o->h.flag & KNH_FLAG_OF_METADATA) == KNH_FLAG_OF_METADATA && ctx != NULL) {
-		TODO();
-//		knh_metadata_clear(ctx, o);
-	}
-#endif
-	if(o->h.bcid == CLASS_Context) return;
 	KNH_OBJECT_REUSE(ctx, o);
 	knh_stat_dclUsedObjectSize(ctx, 1);
 }
@@ -422,17 +406,17 @@ void knh_Object_traverse(Ctx *ctx, knh_Object_t *o, knh_ftraverse ftr)
 {
 	if(IS_SWEEP(ftr)) {
 		knh_Object_free(ctx, o);
-		return;
 	}
-	ctx->share->StructTable[o->h.bcid].ftraverse(ctx, o, ftr);
+	else {
+		StructTable(o->h.bcid).ftraverse(ctx, o, ftr);
+	}
 }
 
-
 /* ========================================================================= */
 
 #define BSHIFT ((KONOHA_PAGESIZE / sizeof(knh_Object_t)) / (sizeof(knh_uintptr_t) * 8))
 
-static int mcount = 0;
+volatile static size_t markedObjectSize = 0;
 
 /* ------------------------------------------------------------------------ */
 
@@ -440,19 +424,21 @@ void knh_Object_mark1(Ctx *ctx, Object *o)
 {
 	if(IS_Context(o)) {
 		//DBG2_P("marked %p, cid=%d,%s", o, knh_Object_cid(o), CLASSN(knh_Object_cid(o)));
-		knh_Object_traverse(ctx, o, knh_Object_mark1);
+		TODO();
+		knh_Context_traverse(ctx, (knh_Context_t*)o, knh_Object_mark1);
 	}
 	else {
 		DBG2_ASSERT(o->h.magic == KNH_OBJECT_MAGIC);
 		DBG2_ASSERT(!knh_isFastMallocMemory((void*)o));
 		knh_uintptr_t *b = (knh_uintptr_t*)((((knh_uintptr_t)o) / KONOHA_PAGESIZE) * KONOHA_PAGESIZE);
+		int n = (o - (Object*)b);
 		b = b + BSHIFT;
-		int n = ((knh_uintptr_t)o % KONOHA_PAGESIZE) / sizeof(knh_Object_t);
-		int x = n/(sizeof(knh_uintptr_t)*8);
-		knh_uintptr_t mask = 1 << (n % (sizeof(knh_uintptr_t)*8));
+		DBG2_ASSERT(n < KONOHA_PAGESIZE / sizeof(knh_Object_t));
+		int x = n / (sizeof(knh_uintptr_t)*8);
+		knh_uintptr_t mask = ((knh_uintptr_t)1) << (n % (sizeof(knh_uintptr_t)*8));
 		if(!((b[x] & mask) == mask)) {
-			//DBG2_P("marked %p, cid=%d,%s", o, knh_Object_cid(o), CLASSN(knh_Object_cid(o)));
-			mcount++;
+			//DBG2_P("marking %p(%d,%d), cid=%d, %s", o, x, n % (sizeof(knh_uintptr_t)*8), knh_Object_cid(o), CLASSN(knh_Object_cid(o)));
+			markedObjectSize++;
 			b[x] = b[x] | mask;
 			knh_Object_traverse(ctx, o, knh_Object_mark1);
 		}
@@ -468,40 +454,36 @@ void knh_Object_finalSweep(Ctx *ctx, Object *o)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method[STATIC] void System.gc() */
 
-METHOD knh__System_gc(Ctx *ctx, knh_sfp_t *sfp)
+void knh_System_gc(Ctx *ctx)
 {
 	int tindex, size = (int)(ctx->share->ObjectPageTableSize);
-
 	DBG_P("** GC - Starting (used %d Kb) ***", (int)ctx->stat->usedMemorySize/1024);
+
 	for(tindex = 0; tindex < size; tindex++) {
 		char *t = ctx->share->ObjectPageTable[tindex].thead;
-
 		if((knh_uintptr_t)t % KONOHA_PAGESIZE != 0) {
 			t = (char*)((((knh_uintptr_t)t / KONOHA_PAGESIZE) + 1) * KONOHA_PAGESIZE);
 			KNH_ASSERT((knh_uintptr_t)t % KONOHA_PAGESIZE == 0);
 		}
 		char *max = ctx->share->ObjectPageTable[tindex].thead + SIZEOF_OBJECTPAGE;
 		while(t + KONOHA_PAGESIZE < max) {
-			knh_uintptr_t *b = (knh_uintptr_t*)t;
-			int i;
-			for(i = 0; i < BSHIFT; i++) {
-				b[i + BSHIFT] = b[i];
-			}
+			knh_memcpy(t + ((BSHIFT) * sizeof(knh_uintptr_t)), t, (BSHIFT) * sizeof(knh_uintptr_t));
+			//knh_bzero(t + ((BSHIFT) * sizeof(knh_uintptr_t)), (BSHIFT) * sizeof(knh_uintptr_t));
 			t += KONOHA_PAGESIZE;
 		}
 	}
 
 	DBG_DISABLE_FREE();
-	mcount = 0;
-	konoha_traverse(ctx, knh_Object_mark1);
+	markedObjectSize = 0;
+	knh_traverseAll(ctx, knh_Object_mark1);
 	DBG_ENABLE_FREE();
-	DBG_P("** GC - Marked %d/%d object(s)", (int)mcount, (int)ctx->stat->usedObjectSize);
+
+	DBG_P("** GC - Marked %d/%d object(s)", (int)markedObjectSize, (int)ctx->stat->usedObjectSize);
 
 	size_t cnt = 0;
 	knh_ftraverse fsweep = ctx->fsweep;
-	((Context*)ctx)->fsweep = knh_Object_finalSweep;
+	((knh_Context_t*)ctx)->fsweep = knh_Object_finalSweep;
 	for(tindex = 0; tindex < size; tindex++) {
 		char *t = ctx->share->ObjectPageTable[tindex].thead;
 
@@ -510,36 +492,28 @@ METHOD knh__System_gc(Ctx *ctx, knh_sfp_t *sfp)
 			KNH_ASSERT((knh_uintptr_t)t % KONOHA_PAGESIZE == 0);
 		}
 		char *max = ctx->share->ObjectPageTable[tindex].thead + SIZEOF_OBJECTPAGE;
-
 		while(t + KONOHA_PAGESIZE < max) {
 			knh_uintptr_t *b = ((knh_uintptr_t*)t) + BSHIFT;
 			int i;
 			for(i = 1; i < (KONOHA_PAGESIZE / sizeof(knh_Object_t)); i++) {
-				int x = i/(sizeof(knh_uintptr_t)*8);
-				knh_uintptr_t mask = 1 << (i % (sizeof(knh_uintptr_t)*8));
+				int x = i / (sizeof(knh_uintptr_t)*8);
+				knh_uintptr_t mask = ((knh_uintptr_t)1) << (i % (sizeof(knh_uintptr_t)*8));
 				if((b[x] & mask) == mask) continue;
 				knh_Object_t *o = ((Object*)t) + i;
 				DBG2_ASSERT(!knh_isFastMallocMemory((void*)o));
+				if(o->h.magic == 0) continue;   // knh_Object_free
+				DBG2_P("%p(%d,%d), magic=%d, cid=%s(%d)", o, i, x, o->h.magic, CLASSN(o->h.bcid), o->h.bcid);
 				knh_Object_RCset(o, 0);
-				if(o->h.magic == 0) continue;
-				//DBG2_P("i=%d, x=%d, o=%p, magic=%d, cid=%s(%d)", i, x, o, o->h.magic, CLASSN(o->h.bcid), o->h.bcid);
 				knh_Object_free(ctx, o);
 				cnt++;
 			}
 			t += KONOHA_PAGESIZE;
 		}
 	}
-	((Context*)ctx)->fsweep = fsweep;
+	((knh_Context_t*)ctx)->fsweep = fsweep;
 	DBG_P("** GC - Collected %d/%d object(s) used=%d Kbytes", (int)cnt, (int)ctx->stat->usedObjectSize, (int)ctx->stat->usedMemorySize/1024);
 }
 
-/* ------------------------------------------------------------------------ */
-
-void knh_System_gc(Ctx *ctx)
-{
-	knh__System_gc(ctx, NULL);
-}
-
 /* ------------------------------------------------------------------------ */
 /* [RCGC] */
 
@@ -553,7 +527,7 @@ void knh_Object_RCsweep(Ctx *ctx, Object *o)
 
 /* ------------------------------------------------------------------------ */
 
-knh_ftraverse konoha_getDefaultSweepFunc()
+knh_ftraverse knh_getDefaultSweepFunc(void)
 {
 	return knh_Object_RCsweep;
 }
